#!/usr/local/bin/perl -w

=head1 NAME

 text2headless.pl

=head1 SYNOPSIS

 text2headless.pl creates a SENSEVAL2 formatted file where the contents 
 of each input file are represented as a seperate context. These 
 contexts do not have any target or head word.

 USAGE: text2headless.pl [OPTIONS] INPUT [[, INPUT] ...]

=head1 DESCRIPTION

 This script converts each given file to a seperate context in SENSEVAL2 
 format.
 It removes all blank-lines and newlines from the input file(s) before
 representing each as a context.
 Any of these contexts do not have any target / head word.
 The file paths are used as the 'docsrc' and the filename or the 
 directory-name are used as the 'senseid'.
 
 This script is usually used for creating 'headless' data for 
 SenseClusters.

 INPUT: Directories and / or files
        eg: text2headless.pl file1 test_dir/

 OUTPUT: SENSEVAL2 formatted file containing each input file as a 
         context at the STDOUT.
         eg:
 <?xml version="1.0" encoding="iso-8859-1" ?>
 <corpus lang='english'>

 <lexelt item="headless">

 <instance id="00001" docsrc="file1">
 <answer instance="00001" senseid="file1"/>
 <context>
 From: un034214@wvnvms.wvnet.eduSubject: NTSC data to RGB ?  For Video 
 Capture. Organization: West Virginia Network for Educational 
 TelecomputingLines: 11Does anyone know how to decode the color 
 information of a NTSC signal ?I need to convert this data to RGB for 
 a Video Capture Utility I am writing for use with an IBM M-MOTION 
 Video adapter card...I need to know the how the V and U signals 
 work in the color process.Thanks in advance for any information 
 or algorythms etc.Later-Hammonck Net
 </context>
 </instance>

 <instance id="00002" docsrc="test_dir/1">
 <answer instance="00002" senseid="test_dir"/>
 <context>
 From: un034214@wvnvms.wvnet.eduSubject: NTSC data to RGB ?  For Video 
 Capture.Organization: West Virginia Network for Educational 
 TelecomputingLines: 11Does anyone know how to decode the color 
 information of a NTSC signal ? I need to convert this data to RGB for a 
 Video Capture Utility I am writing for use with an IBM M-MOTION Video 
 adapter card...I need to know the how the V and U signals work in the 
 color process.Thanks in advance for any information or algorythms 
 etc.Later-Hammonck Net
 </context>
 </instance>

 </lexelt>
 </corpus>

=head1 AUTHOR

Anagha Kulkarni, kulka020@d.umn.edu, University of Minnesota, Duluth

Ted Pedersen, tpederse@d.umn.edu, University of Minnesota, Duluth

=head1 BUGS

=head1 SEE ALSO

 SenseClusters     :  http://sourceforge.net/projects/sensecluster

 Senseval-2        :  http://www.sle.sharp.co.uk/senseval2/  

=head1 COPYRIGHT

Copyright (C) 2004-2006, Ted Pedersen and Anagha Kulkarni

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
USA.

=cut

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

use strict;

use Getopt::Long;

# first check if no commandline options have been provided... in which case
# print out the usage notes!
if ( $#ARGV == -1 )
{
    &minimalUsageNotes();
    exit;
}

# now get the options!
my $result = GetOptions( "version", "help");

our $opt_version;
our $opt_help;

# if help has been requested, print out help!
if ( defined $opt_help )
{
    $opt_help = 1;
    &showHelp();
    exit;
}

# if version has been requested, show version!
if ( defined $opt_version )
{
    $opt_version = 1;
    &showVersion();
    exit;
}

# Program Logic #

my $instanceID = "00001";
my $arg = "";
my $filename = "";
my @filenames = {};

# for each argument:
# 1. check if directory or a file
# 2. if directory glob the filenames under the directory and 
#    send each file to be written as a context at the output
# 3. if file print the contents
# 4. if anything else print error and proceed with next argument.

foreach $arg (@ARGV)
{
    if(-d $arg)
    {
        if($arg=~m/\/$/)
        {
            @filenames = <$arg*>;
        }
        else
        {
            @filenames = <$arg/*>;
        }

        # check every content of the directory:
        # if directory call the recusive function for traversing the directory
        # if file then print the contents as a context
        foreach $filename (@filenames)
        {
            if(-d $filename)
            {
                traverse($filename);
            }
            else
            {
                writecontext($filename, $arg);
            }
        }
    }
    elsif(-f $arg)
    {
        writecontext($arg, $arg);
    }
    else
    {
        print "$arg not a directory or a file name!\n";
    }
}

if($instanceID > 1)
{
    print "</lexelt>\n";
    print "</corpus>\n";
}

# Recursive function 
# checks the given argument - 
# if directory traverse the directory recusively
# if file print the contents as context
sub traverse
{
    my $dir = shift;

    my @filenames = {};
    my $filename ="";

    if($dir=~m/\/$/)
    {
        @filenames = <$dir*>;
    }
    else
    {
        @filenames = <$dir/*>;
    }

    foreach $filename (@filenames)
    {
        if(-d $filename)
        {
            traverse($filename);
        }
        else
        {
            writecontext($filename, $dir);
        }
    }
}

# read the file and write it as a context after
# removing all the newlines and blank lines.
# use the file path as the value for "docsrc"
# and directory name as the senseid

sub writecontext
{
    my $filename = shift;
    my $dirname = shift;
    
    my $senseid = $dirname;
    $senseid =~ s/\/$//;
    
    my $temp_delimiter = $/;
    undef $/;

    open(INP,"<$filename");
    my $file = <INP>;
    
    $/ = $temp_delimiter;
    
    $file =~ s/\n/ /g;
    $file =~ s/^\s+$//g;

    if($instanceID eq "00001")
    {
        # Write the header part to the output file.
        print "<?xml version=\"1.0\" encoding=\"iso-8859-1\" ?>\n";
        print "<corpus lang=\'english\'>\n\n";
        print "<lexelt item=\"headless\">\n\n";
    }
                    
    print "<instance id=\"$instanceID\" docsrc=\"$filename\">\n";
    print "<answer instance=\"".$instanceID++."\" senseid=\"".$senseid."\"/>\n";
    print "<context>\n";
    print "$file\n";
    print "</context>\n";
    print "</instance>\n\n";
    
}


# function to output a minimal usage note when the user has not provided any
# commandline options
sub minimalUsageNotes
{
    print STDERR "Usage: text2headless.pl INPUT [[, INPUT] ...]\n";
    print STDERR "OR text2headless.pl OPTIONS \n";
    askHelp();
}

# function to output help messages for this program
sub showHelp
{
    print "\nUsage: text2headless.pl INPUT [[, INPUT] ...]\n";
    print "\nOR text2headless.pl OPTIONS \n\n";

	print "text2headless.pl creates a SENSEVAL2 formatted file where the \n";
	print "contents of each input file are represented as a seperate context.\n";
	print "These contexts do not have any target or head word.\n\n";

    print "REQUIRED ARGUMENTS:\n\n";
	  
    print "  INPUT                        The input file/directory to be processed.\n";
    print "                               If input is a directory, all text files in\n";
    print "                               it are taken as input files.\n";
	  
    print "OPTIONS:\n\n";
	  
    print "  --version                    Prints the version number.\n\n";
	  
    print "  --help                       Prints this help message.\n\n";
}

# function to output the version number
sub showVersion
{
    print "text2headless.pl      -        version 0.03\n";
    print "Copyright (C) 2004-2006, Ted Pedersen & Anagha Kulkarni\n";
    print "Date of Last Update 10/13/2005\n";
}

# function to output "ask for help" message when the user's goof up!
sub askHelp
{
    print STDERR "Type text2headless.pl --help for help.\n";
}
